;; Generated package description from parsec.el  -*- no-byte-compile: t -*-
(define-package "parsec" "0.1.3.0.20180729.171626" "Parser combinator library" '((emacs "24") (cl-lib "0.5")) :commit "2cbbbc2254aa7bcaa4fb5e07c8c1bf2f381dba26" :authors '(("Junpeng Qiu" . "qjpchmail@gmail.com")) :maintainer '("Junpeng Qiu" . "qjpchmail@gmail.com") :keywords '("extensions") :url "https://github.com/cute-jumper/parsec.el")
